Package: deducorrect
Maintainer: Mark van der Loo <mark.vanderloo@gmail.com>
License: GPL-3
Title: Deductive Correction, Deductive Imputation, and Deterministic
        Correction
Type: Package
LazyLoad: yes
Author: Mark van der Loo, Edwin de Jonge, and Sander
    Scholtus
Description: A collection of methods for automated data cleaning where all actions are logged.
Version: 1.3.7
URL: https://github.com/data-cleaning/deducorrect
Date: 2015-07-15
Depends: editrules (>= 2.9.0)
Imports: stats, utils, methods
NeedsCompilation: no
Packaged: 2015-07-15 06:34:59 UTC; mark
Repository: CRAN
Date/Publication: 2015-07-15 13:23:47
Built: R 4.6.0; ; 2026-04-26 06:52:38 UTC; unix
