# Generated by roxygen2 (4.1.1): do not edit by hand

S3method(as.character,correctionRules)
S3method(correctRounding,editmatrix)
S3method(correctRounding,editset)
S3method(correctSigns,editmatrix)
S3method(correctSigns,editset)
S3method(correctTypos,editmatrix)
S3method(correctTypos,editset)
S3method(correctionRules,character)
S3method(correctionRules,expression)
S3method(deduImpute,editarray)
S3method(deduImpute,editmatrix)
S3method(deduImpute,editset)
S3method(deductiveZeros,editmatrix)
S3method(deductiveZeros,matrix)
S3method(getVars,correctionRules)
S3method(print,correctionRules)
S3method(print,deducorrect)
S3method(solSpace,editmatrix)
S3method(solSpace,matrix)
export(correctRounding)
export(correctSigns)
export(correctTypos)
export(correctWithRules)
export(correctionRules)
export(deduImpute)
export(deductiveLevels)
export(deductiveZeros)
export(imputess)
export(isTotallyUnimodular)
export(solSpace)
import(editrules)
importFrom(methods,new)
importFrom(stats,addmargins)
importFrom(stats,complete.cases)
importFrom(utils,combn)
